
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _CHARACTER_HH_
#define _CHARACTER_HH_

#include <boost/shared_ptr.hpp>

#include <Moof/Entity.hh>
#include <Moof/Event.hh>
#include <Moof/Math.hh>
#include <Moof/RK4.hh>

#include "Animation.hh"
#include "Tilemap.hh"



struct Character;
typedef boost::shared_ptr<Character> CharacterP;


/**
 * Parent class of animate objects with "personalities."  This basically
 * includes the heroine herself and the bad guys.
 */

struct Character : public Mf::Entity
{
	/*
	struct Derivative
	{
		Mf::Vector2 velocity;
		Mf::Vector2 force;

		Derivative operator*(Mf::Scalar dt) const
		{
			Derivative derivative;
			derivative.velocity = dt * velocity;
			derivative.force = dt * force;
			return derivative;
		}

		Derivative operator+(const Derivative& other) const
		{
			Derivative derivative;
			derivative.velocity = velocity + other.velocity;
			derivative.force = force + other.force;
			return derivative;
		}
	};

	struct State
	{
		// primary
		
		Mf::Vector2 position;
		Mf::Vector2 momentum;
		Mf::Vector2 force;

		// secondary

		Mf::Vector2 velocity;

		// constant
		
		Mf::Scalar	mass;
		Mf::Scalar	inverseMass;

		void recalculate()
		{
			velocity = momentum * inverseMass;
		}


		void getDerivative(Derivative& derivative, Mf::Scalar t) const
		{
			//derivative.velocity = Mf::Vector2(0.0, 0.0);
			//derivative.force = Mf::Vector2(0.0, 0.0);
			derivative.velocity = velocity;
			derivative.force = force;

			//Mf::Vector2 x = position - Mf::Vector2(500.0, 200.0);
			//derivative.force += -15.0 * x - 1.5 * velocity;
		}

		void applyDerivative(const Derivative& derivative, Mf::Scalar dt)
		{
			position += dt * derivative.velocity;
			momentum += dt * derivative.force;
			recalculate();
		}

		// these two operator overloads all using the state in generic
		// interpolator implementations

		State operator*(Mf::Scalar scalar) const
		{
			State state = *this;
			state.position *= scalar;
			state.momentum *= scalar;
			state.recalculate();
			return state;
		}

		State operator+(const State& state) const
		{
			State newState = *this;
			newState.position +=  state.position;
			newState.momentum +=  state.momentum;
			newState.recalculate();
			return newState;
		}
	};
*/

	Mf::State2		previous;
	Mf::State2		current;


private:

	static const Mf::Scalar z = 96.0;

protected:

	Mf::Vector2 userForce;

public:

	Character(const std::string& name);
	virtual ~Character() {}

	virtual void update(Mf::Scalar t, Mf::Scalar dt);
	virtual void draw(Mf::Scalar alpha) const;


	Tilemap		tilemap;
	Animation	animation;
};


#endif // _CHARACTER_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

